@@ -317,6 +317,7 @@ class RangeSelection(AbstractController):
 
         self.event_state = "selected"
         self.selection_completed = self.selection
+        self._down_point = []
         event.handled = True
         return
 
@@ -453,12 +454,19 @@ class RangeSelection(AbstractController):
     def selecting_button_up(self, event):
         # Check to see if the selection region is bigger than the minimum
         event.window.set_pointer("arrow")
-        start = self._down_point[self.axis_index]
+
         end = self._get_axis_coord(event)
-        if self.minimum_selection > abs(start - end):
+
+        if len(self._down_point) == 0:
+            cancel_selection = False
+        else:
+            start = self._down_point[self.axis_index]
+            self._down_point = []
+            cancel_selection = self.minimum_selection > abs(start - end)
+
+        if cancel_selection:
             self.deselect(event)
             event.handled = True
-
         else:
             self.event_state = "selected"
 
